package com.battlelancer.seriesguide.ui.episodes;

import android.content.Context;
import android.database.Cursor;
import com.battlelancer.seriesguide.jobs.FlagJobExecutor;
import com.battlelancer.seriesguide.jobs.episodes.EpisodeCollectedJob;
import com.battlelancer.seriesguide.jobs.episodes.EpisodeWatchedJob;
import com.battlelancer.seriesguide.jobs.episodes.EpisodeWatchedUpToJob;
import com.battlelancer.seriesguide.jobs.episodes.SeasonCollectedJob;
import com.battlelancer.seriesguide.jobs.episodes.SeasonWatchedJob;
import com.battlelancer.seriesguide.jobs.episodes.ShowCollectedJob;
import com.battlelancer.seriesguide.jobs.episodes.ShowWatchedJob;
import com.battlelancer.seriesguide.provider.SeriesGuideContract;
import com.battlelancer.seriesguide.util.TimeTools;

public class EpisodeTools {

    /**
     * Checks the database whether there is an entry for this episode.
     */
    public static boolean isEpisodeExists(Context context, int episodeTvdbId) {
        Cursor query = context.getContentResolver().query(
                SeriesGuideContract.Episodes.buildEpisodeUri(episodeTvdbId), new String[] {
                        SeriesGuideContract.Episodes._ID }, null, null, null
        );
        if (query == null) {
            return false;
        }

        boolean isExists = query.getCount() > 0;
        query.close();

        return isExists;
    }

    public static boolean isCollected(int collectedFlag) {
        return collectedFlag == 1;
    }

    public static boolean isSkipped(int episodeFlags) {
        return episodeFlags == EpisodeFlags.SKIPPED;
    }

    public static boolean isUnwatched(int episodeFlags) {
        return episodeFlags == EpisodeFlags.UNWATCHED;
    }

    public static boolean isWatched(int episodeFlags) {
        return episodeFlags == EpisodeFlags.WATCHED;
    }

    public static void validateFlags(int episodeFlags) {
        if (isUnwatched(episodeFlags)) {
            return;
        }
        if (isSkipped(episodeFlags)) {
            return;
        }
        if (isWatched(episodeFlags)) {
            return;
        }

        throw new IllegalArgumentException(
                "Did not pass a valid episode flag. See EpisodeFlags class for details.");
    }

    public static void episodeWatched(Context context, int showTvdbId, int episodeTvdbId,
            int season, int episode, int episodeFlags) {
        validateFlags(episodeFlags);
        FlagJobExecutor.execute(context,
                new EpisodeWatchedJob(showTvdbId, episodeTvdbId, season, episode, episodeFlags));
    }

    public static void episodeCollected(Context context, int showTvdbId, int episodeTvdbId,
            int season, int episode, boolean isCollected) {
        FlagJobExecutor.execute(context,
                new EpisodeCollectedJob(showTvdbId, episodeTvdbId, season, episode, isCollected));
    }

    /**
     * Flags all episodes released up to (== including) this one as watched
     * excluding those with no release date.
     */
    public static void episodeWatchedUpTo(Context context, int showTvdbId,
            long episodeFirstAired, int episodeNumber) {
        FlagJobExecutor.execute(context,
                new EpisodeWatchedUpToJob(showTvdbId, episodeFirstAired, episodeNumber));
    }

    public static void seasonWatched(Context context, int showTvdbId, int seasonTvdbId, int season,
            int episodeFlags) {
        validateFlags(episodeFlags);
        FlagJobExecutor.execute(context, new SeasonWatchedJob(showTvdbId, seasonTvdbId, season,
                episodeFlags, TimeTools.getCurrentTime(context)));
    }

    public static void seasonCollected(Context context, int showTvdbId, int seasonTvdbId,
            int season, boolean isCollected) {
        FlagJobExecutor.execute(context,
                new SeasonCollectedJob(showTvdbId, seasonTvdbId, season, isCollected));
    }

    public static void showWatched(Context context, int showTvdbId, boolean isFlag) {
        FlagJobExecutor.execute(context,
                new ShowWatchedJob(showTvdbId, isFlag ? 1 : 0, TimeTools.getCurrentTime(context)));
    }

    public static void showCollected(Context context, int showTvdbId, boolean isCollected) {
        FlagJobExecutor.execute(context, new ShowCollectedJob(showTvdbId, isCollected));
    }

}
